"use strict";
var websocket = require('websocket');
var W3CWebSocket = websocket['w3cwebsocket'];
var messageTypes_1 = require('./messageTypes');
var protocols_1 = require('./protocols');
var isString = require('lodash.isstring');
var isObject = require('lodash.isobject');
var DEFAULT_SUBSCRIPTION_TIMEOUT = 5000;
var Client = (function () {
    function Client(url, options) {
        var _this = this;
        this.client = new W3CWebSocket(url, protocols_1.GRAPHQL_SUBSCRIPTIONS);
        this.subscriptionHandlers = {};
        this.maxId = 0;
        this.subscriptionTimeout = (options && options.timeout) || DEFAULT_SUBSCRIPTION_TIMEOUT;
        this.waitingSubscriptions = {};
        this.unsentMessagesQueue = [];
        this.client.onopen = function () {
            _this.unsentMessagesQueue.forEach(function (message) {
                _this.client.send(JSON.stringify(message));
            });
            _this.unsentMessagesQueue = [];
        };
        this.client.onmessage = function (message) {
            var parsedMessage;
            try {
                parsedMessage = JSON.parse(message.data);
            }
            catch (e) {
                throw new Error('Message must be JSON-parseable.');
            }
            var subId = parsedMessage.id;
            if (parsedMessage.type !== messageTypes_1.SUBSCRIPTION_KEEPALIVE && !_this.subscriptionHandlers[subId]) {
                _this.unsubscribe(subId);
                return;
            }
            switch (parsedMessage.type) {
                case messageTypes_1.SUBSCRIPTION_SUCCESS:
                    delete _this.waitingSubscriptions[subId];
                    break;
                case messageTypes_1.SUBSCRIPTION_FAIL:
                    if (_this.subscriptionHandlers[subId]) {
                        _this.subscriptionHandlers[subId](_this.formatErrors(parsedMessage.payload.errors), null);
                    }
                    delete _this.subscriptionHandlers[subId];
                    delete _this.waitingSubscriptions[subId];
                    break;
                case messageTypes_1.SUBSCRIPTION_DATA:
                    if (parsedMessage.payload.data && !parsedMessage.payload.errors) {
                        _this.subscriptionHandlers[subId](null, parsedMessage.payload.data);
                    }
                    else {
                        _this.subscriptionHandlers[subId](_this.formatErrors(parsedMessage.payload.errors), null);
                    }
                    break;
                case messageTypes_1.SUBSCRIPTION_KEEPALIVE:
                    break;
                default:
                    throw new Error('Invalid message type - must be of type `subscription_start`, `subscription_data` or `subscription_keepalive`.');
            }
        };
    }
    Client.prototype.subscribe = function (options, handler) {
        var _this = this;
        var query = options.query, variables = options.variables, operationName = options.operationName, context = options.context;
        if (!query) {
            throw new Error('Must provide `query` to subscribe.');
        }
        if (!handler) {
            throw new Error('Must provide `handler` to subscribe.');
        }
        if (!isString(query) ||
            (operationName && !isString(operationName)) ||
            (variables && !isObject(variables))) {
            throw new Error('Incorrect option types to subscribe. `subscription` must be a string,' +
                '`operationName` must be a string, and `variables` must be an object.');
        }
        var subId = this.generateSubscriptionId();
        var message = Object.assign(options, { type: messageTypes_1.SUBSCRIPTION_START, id: subId });
        this.sendMessage(message);
        this.subscriptionHandlers[subId] = handler;
        this.waitingSubscriptions[subId] = true;
        setTimeout(function () {
            if (_this.waitingSubscriptions[subId]) {
                handler([new Error('Subscription timed out - no response from server')]);
                _this.unsubscribe(subId);
            }
        }, this.subscriptionTimeout);
        return subId;
    };
    Client.prototype.unsubscribe = function (id) {
        delete this.subscriptionHandlers[id];
        var message = { id: id, type: messageTypes_1.SUBSCRIPTION_END };
        this.sendMessage(message);
    };
    Client.prototype.unsubscribeAll = function () {
        var _this = this;
        Object.keys(this.subscriptionHandlers).forEach(function (subId) {
            _this.unsubscribe(parseInt(subId));
        });
    };
    Client.prototype.sendMessage = function (message) {
        switch (this.client.readyState) {
            case this.client.OPEN:
                this.client.send(JSON.stringify(message));
                break;
            case this.client.CONNECTING:
                this.unsentMessagesQueue.push(message);
                break;
            case this.client.CLOSING:
            case this.client.CLOSED:
            default:
                throw new Error('Client is not connected to a websocket.');
        }
    };
    Client.prototype.generateSubscriptionId = function () {
        var id = this.maxId;
        this.maxId += 1;
        return id;
    };
    Client.prototype.formatErrors = function (errors) {
        if (Array.isArray(errors)) {
            return errors;
        }
        if (errors && errors.message) {
            return [errors];
        }
        return [{ message: 'Unknown error' }];
    };
    return Client;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Client;
;
//# sourceMappingURL=client.js.map